/*
 * This file is a part of the CaosDB Project.
 *
 * Copyright (C) 2021 Timm Fitschen <t.fitschen@indiscale.com>
 * Copyright (C) 2021 Florian Spreckelsen <f.spreckelsen@indiscale.com>
 * Copyright (C) 2021 IndiScale GmbH <info@indiscale.com>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program. If not, see <https://www.gnu.org/licenses/>.
 *
 */
#include "caosdb/entity.h"
#include "caosdb/entity/v1alpha1/main.pb.h" // for RepeatedPtrField, Property
#include "caosdb/protobuf_helper.h"         // for get_arena
#include <google/protobuf/arena.h>          // for Arena
#include <new>                              // for operator new

namespace caosdb::entity {
using caosdb::entity::v1alpha1::IdResponse;
using ProtoParent = caosdb::entity::v1alpha1::Parent;
using ProtoProperty = caosdb::entity::v1alpha1::Property;
using ProtoEntity = caosdb::entity::v1alpha1::Entity;
using ProtoMessage = caosdb::entity::v1alpha1::Message;
using ProtoFileDescriptor = caosdb::entity::v1alpha1::FileDescriptor;
using caosdb::utility::get_arena;
using google::protobuf::Arena;

Messages::~Messages() = default;

Parent::Parent() : wrapped(Parent::CreateProtoParent()) {
  // TODO(fspreck) Re-enable once we have decided how to attach
  // messages to parents.
  // errors.wrapped = this->wrapped->mutable_errors();
  // warnings.wrapped = this->wrapped->mutable_warnings();
  // infos.wrapped = this->wrapped->mutable_infos();
}

auto Parent::CreateProtoParent() -> ProtoParent * {
  return Arena::CreateMessage<ProtoParent>(get_arena());
}

auto Parent::SetName(const std::string &name) -> void {
  this->wrapped->set_name(name);
}

auto Parent::SetId(const std::string &id) -> void { this->wrapped->set_id(id); }

[[nodiscard]] auto Parent::GetId() const -> const std::string & {
  return this->wrapped->id();
}

[[nodiscard]] auto Parent::GetName() const -> const std::string & {
  return this->wrapped->name();
}

[[nodiscard]] auto Parent::GetDescription() const -> const std::string & {
  return this->wrapped->description();
}

Property::Property() : wrapped(Property::CreateProtoProperty()) {}

auto Property::CreateProtoProperty() -> ProtoProperty * {
  return Arena::CreateMessage<ProtoProperty>(get_arena());
}

[[nodiscard]] auto Property::GetId() const -> const std::string & {
  return this->wrapped->id();
}

[[nodiscard]] auto Property::GetName() const -> const std::string & {
  return this->wrapped->name();
}

[[nodiscard]] auto Property::GetDescription() const -> const std::string & {
  return this->wrapped->description();
}

[[nodiscard]] auto Property::GetImportance() const -> const std::string & {
  return this->wrapped->importance();
}

[[nodiscard]] auto Property::GetValue() const -> const std::string & {
  return this->wrapped->value();
}

[[nodiscard]] auto Property::GetUnit() const -> const std::string & {
  return this->wrapped->unit();
}

[[nodiscard]] auto Property::GetDatatype() const -> const std::string & {
  return this->wrapped->datatype();
}

auto Property::SetId(const std::string &id) -> void {
  this->wrapped->set_id(id);
}

auto Property::SetName(const std::string &name) -> void {
  this->wrapped->set_name(name);
}

auto Property::SetDescription(const std::string &description) -> void {
  this->wrapped->set_description(description);
}

auto Property::SetImportance(const std::string &importance) -> void {
  this->wrapped->set_importance(importance);
}

auto Property::SetValue(const std::string &value) -> void {
  this->wrapped->set_value(value);
}

auto Property::SetUnit(const std::string &unit) -> void {
  this->wrapped->set_unit(unit);
}

auto Property::SetDatatype(const std::string &datatype) -> void {
  this->wrapped->set_datatype(datatype);
}

[[nodiscard]] auto Entity::GetParents() const -> const Parents & {
  return parents;
}

auto Entity::AppendParent(const Parent &parent) -> void {
  this->parents.Append(parent);
}

auto Entity::RemoveParent(int index) -> void { this->parents.remove(index); }

[[nodiscard]] auto Entity::GetProperties() const -> const Properties & {
  return properties;
}

auto Entity::AppendProperty(const Property &property) -> void {
  this->properties.Append(property);
}

auto Entity::RemoveProperty(int index) -> void {
  this->properties.remove(index);
}

auto Entity::CreateProtoEntity() -> ProtoEntity * {
  return Arena::CreateMessage<ProtoEntity>(get_arena());
}

auto Entity::CreateMessagesField() -> RepeatedPtrField<ProtoMessage> * {
  return Arena::CreateMessage<RepeatedPtrField<ProtoMessage>>(get_arena());
}

Entity::Entity() : Entity(Entity::CreateProtoEntity()) {}

Entity::Entity(IdResponse *idResponse) : Entity() {
  this->wrapped->set_id(idResponse->id());
  this->errors.wrapped->Swap(idResponse->mutable_errors());
  this->warnings.wrapped->Swap(idResponse->mutable_warnings());
  this->infos.wrapped->Swap(idResponse->mutable_infos());
}

auto Entity::SetId(const std::string &id) -> void { this->wrapped->set_id(id); }

auto Entity::SetVersionId(const std::string &id) -> void {
  this->wrapped->mutable_version()->set_id(id);
}

auto Entity::CopyTo(ProtoEntity *target) -> void {
  target->CopyFrom(*(this->wrapped));
}

auto Entity::SetRole(const std::string &role) -> void {
  this->wrapped->set_role(role);
}

auto Entity::SetName(const std::string &name) -> void {
  this->wrapped->set_name(name);
}

auto Entity::SetDescription(const std::string &description) -> void {
  this->wrapped->set_description(description);
}

auto Entity::SetValue(const std::string &value) -> void {
  this->wrapped->set_value(value);
}

auto Entity::SetUnit(const std::string &unit) -> void {
  this->wrapped->set_unit(unit);
}

auto Entity::SetDatatype(const std::string &datatype) -> void {
  this->wrapped->set_datatype(datatype);
}

auto Entity::SetFilePath(const std::string &path) -> void {
  this->wrapped->mutable_file_descriptor()->set_path(path);
}

} // namespace caosdb::entity
